/******************************************
 * REAL TIME ANAGLYPH CAPTURE             *
 * Tutor: Paolo Pasteris                  *
 * Progetto multimediale 12/13 Unimi      *
 ******************************************/

import codeanticode.gsvideo.*;

GSCapture camSx, camDx;
PImage img;
GSMovieMaker mm;
int fps, pixSx, pixDx, disparityX, disparityY, newX, newY;
PFont font;

void setup() {
  // sketch initialization
  fps = 25;
  size(320, 240);
  frameRate(fps);
  // camera initialization
  //String[] cameras = GSCapture.list();     // doesn't work on mac
  camSx = new GSCapture(this, width, height, "USB Video Class Video:0");
  camDx = new GSCapture(this, width, height, "USB Video Class Video:1");
  
  camSx.start();
  camDx.start();
  
  img = new PImage(width, height);
  // video encoder initialization
  mm = new GSMovieMaker(this, width, height, "anaglyph.ogg",
                        GSMovieMaker.THEORA, GSMovieMaker.HIGH,
                        fps);
  disparityX = 0;
  disparityY = 0;
  font = loadFont("DejaVuSans-12.vlw");
  textFont(font);
}

void draw() {
  //if (camSx.available() && camDx.available()) {
    // read from both cameras
    camSx.read();
    camDx.read();
    for (int x = 0; x < width; x++) {
      for (int y = 0; y < height; y++) {
        // bitmasking pixel (X,Y) of left image
        // (saving just the red channel)
        pixSx = camSx.get(x,y) & 0x00FF0000;
        // selecting right pixel position, based on disparity
        newX = x + disparityX;
        newY = y + disparityY;
        // bitmasking pixel (newX,newY) of right image
        // (saving green and blue channels)
        // defaulting to black if (newX,newY) is not in the frame
        pixDx = newX >= 0 && newX < width &&
                newY >= 0 && newY < height ? 
                camDx.get(newX,newY) & 0x0000FFFF :
                0;
        // adding the masked pixels to the stereoscopic image
        img.set(x,y,pixSx+pixDx);
      }
    //}
    // display the stereoscopic image
    image(img,0,0);
  }
  // add frame to the video, if recording
  if (mm.isRecording()) mm.addFrame(img.pixels);
  
  // print instructions on screen
  fill(255,255,0);
  text("ARROW KEYS: control disparity\n" +
       "SPACE: start/stop recording", 10, 16);
  if (mm.isRecording()) text("RECORDING", 10, height - 6);
}

void keyPressed() {
  if (key == CODED) {
    // control disparity
    if (keyCode == LEFT) {
      disparityX++;
      return;
    }
    if (keyCode == RIGHT) {
      disparityX--;
      return;
    }
    if (keyCode == UP) {
      disparityY++;
      return;
    }
    if (keyCode == DOWN) {
      disparityY--;
      return;
    }
  }
  // start/stop recording
  if (key == ' ') {
    if (mm.isRecording()) mm.finish();
    else mm.start();
  }
}
